/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File stub_client.c */
/* This is a stub client (user interface) thread used to test the
 * communication mechanisms between threads. */


#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>
#include <termios.h>

#include "lgd0.h"
#include "lgdlib.h"
#include "fifo.h"

#define COLUMNS 80
#define LINES 24

char redraw_needed;

void *
output_main(void *garbage)
{
  msg_fifo_t msg;
  msg_response_t resp;
  msg.treated=1;
  while (1) {
    int j,i;
    redraw_needed=0;
    for (j=0;j<LINES;j++) {
      printf("\033[%dH",j+1);
      for (i=0;i<COLUMNS;i++) {
	while (!msg.treated) sched_yield(); /* This shouldn't happen */
	msg.in_heap=0;
	msg.response=&resp;
	msg.type=msg_readmap;
	msg.param.readmap.x=i;
	msg.param.readmap.y=j;
	resp.ready=0;
	append_control_msg(&msg);
	while (!resp.ready) sched_yield();
	switch (resp.ret.readmap.val) {
	case 0: printf("."); break;
	case 1: printf(" "); break;
	case 2: printf("@"); break;
	}
      }
    }
    fflush(stdout);
    while (!redraw_needed) sched_yield();
  }
}

void *
input_main(void *garbage)
{
  struct termios termios_0,termios_1;
  msg_fifo_t msg;
  if (tcgetattr(0,&termios_0))
    lgd_fatal("Can't read terminal attributes.");
  termios_1=termios_0;
  termios_1.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
  if (tcsetattr(0,TCSANOW,&termios_1))
    lgd_fatal("Can't set terminal attributes.");
  msg.treated=1;
  while (1) {
    char ch;
    ch=getchar();
    if ((ch=='h')||(ch=='j')||(ch=='k')||(ch=='l')) {
      while (!msg.treated) sched_yield(); /* User typing too fast! */
      msg.in_heap=0;
      msg.response=NULL; /* We don't care whether things fail */
      msg.type=msg_move_hero;
      switch (ch) {
      case 'h':
	msg.param.move_hero.dx=-1;
	msg.param.move_hero.dy=0;
	break;
      case 'j':
	msg.param.move_hero.dx=0;
	msg.param.move_hero.dy=1;
	break;
      case 'k':
	msg.param.move_hero.dx=0;
	msg.param.move_hero.dy=-1;
	break;
      case 'l':
	msg.param.move_hero.dx=1;
	msg.param.move_hero.dy=0;
	break;
      }
      append_control_msg(&msg);
      redraw_needed=1;
    }
  }
}
