/* This file is part of Legendes II */
/* Copyright (c) 1998 by
 *   Philippe Clavel (philippe.clavel@supelec.fr)
 *   David Madore (david.madore@ens.fr) */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* File stub_server.c */
/* This is a stub server thread used to test the communication
 * mechanisms between threads. */

#include <pthread.h>

#include "lgd0.h"
#include "lgdlib.h"
#include "fifo.h"

#define HSIZE  80
#define VSIZE  24

static int map[HSIZE][VSIZE];

typedef struct hero_s {
  int x;
  int y;
} hero_t;

static hero_t hero;

void
server_init (void)
{
  int i,j;
  for (i=0;i<HSIZE;i++) for (j=0;j<VSIZE;j++)
    map[i][j] = 0;
  for (i=35;i<50;i++) for (j=5;j<10;j++)
    map[i][j] = 1;
  hero.x = 5;
  hero.y = 5;
}
     
void *
server_main(void *garbage)
{
  server_init();
  while (1) {
    msg_fifo_t *crtmsg;
    msg_response_t *resp;
    crtmsg = process_control_msg();
    if (crtmsg != NULL) {
      resp = crtmsg->response;
      switch (crtmsg->type) {
      case msg_readmap : {
	int x,y;
	if (resp) {
	  x = crtmsg->param.readmap.x;
	  y = crtmsg->param.readmap.y;
	  if ((hero.x==x)&&(hero.y==y))
	    resp->ret.readmap.val = 2;
	  else if (map[x][y] == 0)
	    resp->ret.readmap.val = 0;
	  else if (map[x][y] == 1)
	    resp->ret.readmap.val = 1;
	  resp->errcode = 0;
	}
	} break;
      case msg_move_hero : {
	int xf,yf;
	xf = crtmsg->param.move_hero.dx + hero.x;
	yf = crtmsg->param.move_hero.dy + hero.y;
	if ((xf>=0)&&(xf<HSIZE)&&(yf>=0)&&(yf<VSIZE)&&(map[xf][yf]==0)) {
	  hero.x = xf;
	  hero.y = yf;
	  if (resp) resp->errcode = 0;
	} else {
	  if (resp) resp->errcode = 1; /* BONK! */
	}
	} break;
      default :
	if (resp) resp->errcode = 2;
	break;
      }
      if (resp) resp->ready=1;
      if (crtmsg->in_heap)
	lgd_free(crtmsg);
      else crtmsg->treated=1;
    } else sched_yield();
  }
}
